﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AsReaderBox_Demo_Tool
{
    public enum PadType
    {
        PAD_LEFT,
        PAD_RIGHT
    }

    public class ByteBuilder
    {
        private readonly int MAX_CAPACITY = (256 * 256);
        private readonly int DEFAULT_CAPACITY = (256);
        private byte[] m_ArrayBytes;
        private int m_nCapacity;
        private int m_nPos;

        /*
        public static bool ByteArrayCompare(byte[] a1, byte[] a2)
        {
            IStructuralEquatable eqa1 = a1;
            return eqa1.Equals(a2, StructuralComparisons.StructuralEqualityComparer);
        } 
        */

        public ByteBuilder(int nCap)
        {
            m_nCapacity = nCap;
            m_ArrayBytes = new byte[m_nCapacity];
            m_nPos = 0;
        }

        public ByteBuilder(byte[] byteArray)
        {
            if (byteArray == null)
                return;
            m_nCapacity = byteArray.Length;
            m_ArrayBytes = byteArray;
            m_nPos = byteArray.Length;
        }

        public ByteBuilder()
        {
            m_nCapacity = DEFAULT_CAPACITY;
            m_ArrayBytes = new byte[m_nCapacity];
            m_nPos = 0;
        }

        public void Append(sbyte b)
        {
            if (m_nPos < Capacity - 2)
            {
                m_ArrayBytes[m_nPos++] = (byte)b;
            }
            else
            {
                Capacity += DEFAULT_CAPACITY;
                m_ArrayBytes[m_nPos++] = (byte)b;
            }

        }

        public void Append(byte b)
        {
            if (m_nPos < Capacity - 2)
            {
                m_ArrayBytes[m_nPos++] = b;
            }
            else
            {
                Capacity += DEFAULT_CAPACITY;
                m_ArrayBytes[m_nPos++] = b;
            }

        }

        public void Append(byte[] ba)
        {
            if (ba == null || ba.Length == 0) return;

            for (int i = 0; i < ba.Length; i++)
            {
                this.Append((byte)ba[i]);
            }

        }

        public void Append(byte[] ba, int length)
        {
            if (ba == null || ba.Length == 0) return;

            for (int i = 0; i < length; i++)
            {
                this.Append((byte)ba[i]);
            }

        }

        public void Append(UInt16 val)
        {
            byte[] ba = BitConverter.GetBytes(val);

            if (BitConverter.IsLittleEndian)
            {
                Array.Reverse(ba);
            }

            Append(ba);
        }
        public void Append(string s)
        {
            Byte[] sendBytes = Encoding.ASCII.GetBytes(s);
            Append(sendBytes);
        }

        public int Length
        {
            get
            {
                return m_nPos;
            }
        }

        public int Capacity
        {
            get
            {
                return m_nCapacity;
            }
            set
            {
                if (value < 0)
                    throw new ArgumentOutOfRangeException("value", "ArgumentOutOfRange NegativeCapacity");

                if (value < this.m_ArrayBytes.Length)
                {
                    throw new ArgumentOutOfRangeException("value", "ArgumentOutOfRange SmallCapacity");
                }
                if (value > this.MAX_CAPACITY)
                {
                    throw new ArgumentOutOfRangeException("value", "ArgumentOutOfRange Capacity");
                }

                int currentCapacity = this.m_ArrayBytes.Length;
                if (value != currentCapacity)
                {
                    Array.Resize(ref m_ArrayBytes, value);
                }

                m_nCapacity = value;
            }
        }

        public void Clear()
        {
            m_nCapacity = DEFAULT_CAPACITY;
            m_ArrayBytes = new byte[m_nCapacity];
            m_nCapacity = DEFAULT_CAPACITY;
            m_nPos = 0;
        }

        override public string ToString()
        {
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < m_nPos; i++)
            {
                sb.AppendFormat(m_ArrayBytes[i].ToString("X2"));
            }
            return sb.ToString();
        }

        public string ToString(string format)
        {
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < m_nPos; i++)
            {
                sb.AppendFormat(m_ArrayBytes[i].ToString(format));
            }
            return sb.ToString();
        }

        public byte[] GetByteArray()
        {
            byte[] ret = new byte[m_nPos];
            Array.Copy(m_ArrayBytes, ret, m_nPos);
            return ret;
        }

        public byte[] GetByteArray(int index, int length)
        {
            byte[] ret = new byte[length];
            //           Array.Copy(m_ArrayBytes, ret, m_nPos);
            Array.Copy(m_ArrayBytes, index, ret, 0, length);
            return ret;
        }

        //20110216 Add Function (by sjpark)s
        public byte GetAt(int index)
        {

            return m_ArrayBytes[index];

        }
    }

    class AsreaderHelper
    {
        public static string ByteArrayToHexString2(byte[] data)
        {
            StringBuilder sb = new StringBuilder(data.Length * 3);
            foreach (byte b in data)
                sb.Append(Convert.ToString(b, 16).PadLeft(2, '0').PadLeft(3, ' '));
            return sb.ToString().ToUpper();

        }

        public static string ByteArrayToHexString(byte[] data)
        {
            StringBuilder sb = new StringBuilder(data.Length * 3);
            foreach (byte b in data)
                sb.Append(Convert.ToString(b, 16).PadLeft(2, '0'));
            return sb.ToString().ToUpper();
        }


        static public byte[] HexStringToByteArray(string s)
        {
            s = s.Replace(" ", "").Replace("0x", "");
            byte[] buffer = new byte[s.Length / 2];
            for (int i = 0; i < s.Length; i += 2)
                buffer[i / 2] = (byte)Convert.ToByte(s.Substring(i, 2), 16);
            return buffer;
        }

        static public int HexStringToInt(string s)
        {
            return Convert.ToInt32(s, 16);
        }

        static public int DecStringToInt(string s)
        {
            return Convert.ToInt32(s, 10);
        }

        static public string ByteArrayToHexString(byte[] byteArray, int length, bool addSpace)
        {
            StringBuilder sb = new StringBuilder();

            int len = length;
            if (byteArray.Length < len)
                len = byteArray.Length;

            for (int i = 0; i < len; i++)
            {
                sb.Append(byteArray[i].ToString("X02"));

                if (addSpace) sb.Append(" ");
            }

            return sb.ToString();
        }

        static public string ArgByteToStringByte(byte[] byteArray)
        {
            StringBuilder sb = new StringBuilder();

            foreach (byte abyte in byteArray)
            {
                sb.Append(abyte.ToString("X02") + " ");
            }

            return sb.ToString();
        }

        static public string ArgByteToStringByte2(byte[] byteArray)
        {
            StringBuilder sb = new StringBuilder();

            foreach (byte abyte in byteArray)
            {
                sb.Append(abyte.ToString("X02"));
            }

            return sb.ToString();
        }

        static public byte[] ArgStringHexToByte(string val)
        {
            ByteBuilder bb = new ByteBuilder();
            char[] delimStr = { ' ' };

            val = val.Trim();
            val = val.Replace("0x", "");
            val = val.Replace(" ", "");

            try
            {
                for (int i = 0; i < val.Length / 2; i++)
                {
                    bb.Append(Convert.ToByte(val.Substring(i * 2, 2), 16));
                }
            }
            catch
            {
            }

            return bb.GetByteArray();
        }

        static public byte[] ArgStringBinaryToByte(string val, PadType pad)
        {
            ByteBuilder bb = new ByteBuilder();
            char[] delimStr = { ' ' };

            val = val.Trim();
            val = val.Replace(" ", "");
            if (pad == PadType.PAD_LEFT)
            {
                val = val.PadLeft(((val.Length + 7) >> 3) * 8, '0');
            }
            else
            {
                val = val.PadRight(((val.Length + 7) >> 3) * 8, '0');
            }

            try
            {
                for (int i = 0; i < (val.Length + 7) >> 3; i++)
                {
                    if (i == 0) bb.Append(Convert.ToByte(val.Substring(i * 8, 8 - val.Length % 8), 2));
                    else bb.Append(Convert.ToByte(val.Substring(i * 8 - val.Length % 8, 8), 2));
                }
            }
            catch
            {

            }

            return bb.GetByteArray();
        }

        static public string ArgByteToStringBinary(byte[] val, int BitLength)
        {
            int len;
            StringBuilder sb = new StringBuilder();

            if (BitLength > (val.Length + 7 / 8)) len = (val.Length + 7 / 8);
            else len = BitLength;

            for (int i = 0; i < len; i++)
            {
                for (int j = 0; j < 8; j++)
                {
                    if (((val[i] >> (7 - j)) & 0x01) == 0x01) sb.Append("1");
                    else sb.Append("0");
                }
                sb.Append(" ");
            }

            return sb.ToString();
        }
    }
}
